# -*- coding: utf-8 -*-
"""This file imports Python modules that register analysis plugins."""

from plaso.analysis import browser_search
from plaso.analysis import chrome_extension
from plaso.analysis import file_hashes
from plaso.analysis import nsrlsvr
from plaso.analysis import sessionize
from plaso.analysis import tagging
from plaso.analysis import unique_domains_visited
from plaso.analysis import viper
from plaso.analysis import virustotal
from plaso.analysis import windows_services
