# -*- coding: utf-8 -*-
"""Compound ZIP plugin related functions and classes for testing."""

from __future__ import unicode_literals

from tests.parsers import test_lib


class CompoundZIPPluginTestCase(test_lib.ParserTestCase):
  """The unit test case for a compound ZIP plugin."""
